{% extends "base.html" %}
{% block content %}
<h2 class="sub-header">Jobber log files</h2>
<div id="log_files_row" class="row top-buffer col-md-12">
	<button id="main_log_btn" name="button2id" class="btn btn-success custombtn" title="Log file">Show Main LOG file</button>
	<div id="name" class="row top-buffer">
		<div id="main_log" class="col-sm-12" style="display:none">
			<div id="out_row" class="row top_buffer col-sm-12">
				<h4>Main log file:</h4>
				<div id="out_file" class="col-sm-6" style="background-color:#87CEFA; border-radius: 10px;">
				{{ main_log | safe}}
				</div>
			</div>
		</div>
	</div>
</div>

<div id="log_files_row" class="row top-buffer col-md-12">
	<button id="handler_log_btn" name="button2id" class="btn btn-primary custombtn" title="Log file">Show MessageHandler LOG file</button>
	<div id="name" class="row top-buffer">
		<div id="handler_log" class="col-sm-12" style="display:none">
			<div id="out_row" class="row top_buffer col-sm-12">
				<h4>MessageHandler log file:</h4>
				<div id="out_file" class="col-sm-6" style="background-color:#87CEFA; border-radius: 10px;">
				{{ handler_log | safe}}
				</div>
			</div>
		</div>
	</div>
</div>

<div id="log_files_row" class="row top-buffer col-md-12">
	<button id="controller_log_btn" name="button2id" class="btn btn-primary custombtn" title="Log file">Show JobController LOG file</button>
	<div id="name" class="row top-buffer">
		<div id="controller_log" class="col-sm-12" style="display:none">
			<div id="out_row" class="row top_buffer col-sm-12">
				<h4>JobController file:</h4>
				<div id="out_file" class="col-sm-6" style="background-color:#87CEFA; border-radius: 10px;">
				{{ controller_log | safe}}
				</div>
			</div>
		</div>
	</div>
</div>

<div id="log_files_row" class="row top-buffer col-md-12">
	<button id="executer_log_btn" name="button2id" class="btn btn-primary custombtn" title="Log file">Show DRMAAJobExecuter LOG file</button>
	<div id="name" class="row top-buffer">
		<div id="executer_log" class="col-sm-12" style="display:none">
			<div id="out_row" class="row top_buffer col-sm-12">
				<h4>DRMAAJobExecuter file:</h4>
				<div id="out_file" class="col-sm-6" style="background-color:#87CEFA; border-radius: 10px;">
				{{ executer_log | safe}}
				</div>
			</div>
		</div>
	</div>
</div>
{% endblock %}
