<!DOCTYPE html>
<html lang="en">
	<head>
		<meta http-equiv="content-type" content="text/html; charset=UTF-8">
		<meta charset="utf-8">
		<title>Dashboard for Jobber</title>
		<meta name="generator" content="Bootply" />
		<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
	{% block css %}
	<link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/bootstrap.min.css') }}" />
  <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/bootstrap-table.css') }}" />
	<link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/styles.css') }}" />
	<link rel="shortcut icon" href="{{ url_for('static', filename='images/favicon.png')  }}">
    {% endblock %}
		<!--[if lt IE 9]>
			<script src="//html5shim.googlecode.com/svn/trunk/html5.js"></script>
		<![endif]-->
	</head>
	<body>
<nav class="navbar navbar-inverse navbar-fixed-top" role="navigation">
      <div class="container-fluid">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="/">Jobber</a>
        </div>
        <div class="navbar-collapse collapse">
          <ul class="nav navbar-nav navbar-right">
            <li><a href="/">Dashboard</a></li>
            <!-- <li><a href="#">Links</a></li>
            <li><a href="#">Help</a></li> -->
          </ul>
          <form class="navbar-form navbar-right">
            <input type="text" class="form-control" placeholder="Search...">
          </form>
        </div>
      </div>
</nav>

<div class="container-fluid">
      
      <div class="row row-offcanvas row-offcanvas-left">
        
         <div class="col-sm-3 col-md-2 sidebar-offcanvas" id="sidebar" role="navigation">
           
            <ul class="nav nav-sidebar">
              <!-- <li class="active"><a href="#">Overview</a></li> -->
              <li><a href="/">Pipelines</a></li>
              <li><a href="/logs">Logs</a></li>
              <!-- <li><a href="#">Jobs</a></li> -->
            </ul>
          
        </div><!--/span-->
        
        <div class="col-sm-9 col-md-10 main">
          
          <!--toggle sidebar button-->
          <p class="visible-xs">
            <button type="button" class="btn btn-primary btn-xs" data-toggle="offcanvas"><i class="glyphicon glyphicon-chevron-left"></i></button>
          </p>
          
		  <h1 class="page-header">
            Dashboard for Jobber
          </h1>

<!--           <div class="row placeholders">
            <div class="col-xs-6 col-sm-3 placeholder text-center">
              <img src="//placehold.it/200/6666ff/fff" class="center-block img-responsive img-circle" alt="Generic placeholder thumbnail">
              <h4>Label</h4>
              <span class="text-muted">Something else</span>
            </div>
            <div class="col-xs-6 col-sm-3 placeholder text-center">
              <img src="//placehold.it/200/66ff66/fff" class="center-block img-responsive img-circle" alt="Generic placeholder thumbnail">
              <h4>Label</h4>
              <span class="text-muted">Something else</span>
            </div>
            <div class="col-xs-6 col-sm-3 placeholder text-center">
              <img src="//placehold.it/200/6666ff/fff" class="center-block img-responsive img-circle" alt="Generic placeholder thumbnail">
              <h4>Label</h4>
              <span class="text-muted">Something else</span>
            </div>
            <div class="col-xs-6 col-sm-3 placeholder text-center">
              <img src="//placehold.it/200/66ff66/fff" class="center-block img-responsive img-circle" alt="Generic placeholder thumbnail">
              <h4>Label</h4>
              <span class="text-muted">Something else</span>
            </div>
          </div> -->

    {% block content %}
        <p>Use this document as a way to quick start any new project.</p>
    {% endblock content %}

      </div><!--/row-->
	</div>
</div><!--/.container-->

<footer>
  <p class="pull-right">©2015 University of Basel, Biozentrum | Swiss Institute of Bioinformatics</p>
</footer>
        
	<!-- script references -->
	{% block javascript %}
  <script src='http://ajax.googleapis.com/ajax/libs/jquery/1.11.2/jquery.min.js'></script>
      <script src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
      <script src="{{ url_for('static', filename='js/bootstrap-table.js') }}"></script>
      <script src="{{ url_for('static', filename='js/scripts.js') }}"></script>
      <script src="{{ url_for('static', filename='js/tooltip.js') }}"></script>
      <script src="{{ url_for('static', filename='js/alert.js') }}"></script>
      <script src="{{ url_for('static', filename='js/modal.js') }}"></script>
      <script src="{{ url_for('static', filename='js/transition.js') }}"></script>
    {% endblock javascript %}
	</body>
</html>
